package cd.casic.ci.common.pipeline.container;

import cd.casic.ci.common.pipeline.NameAndValue;
import cd.casic.ci.common.pipeline.option.JobControlOption;
import cd.casic.ci.common.pipeline.option.MatrixControlOption;
import io.swagger.v3.oas.annotations.media.Schema;
import kotlin.ReplaceWith;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.util.List;
import java.util.Map;

@Data
@EqualsAndHashCode(callSuper = false)
@Schema(title = "流水线模型-普通任务容器")
public class NormalContainer extends Container {
    public static final String classType = "normal";

    private Boolean enableSkip = false;

    private List<NameAndValue> conditions;

    private Integer maxQueueMinutes = 60;

    private Integer maxRunningMinutes = 1440;

    private JobControlOption jobControlOption; // 为了兼容旧数据，所以定义为可空以及var
    private MutexGroup mutexGroup; // 为了兼容旧数据，所以定义为可空以及var
    private MatrixControlOption matrixControlOption;
    private String matrixGroupId;
    private Map<String, String> matrixContext;
    private List<NormalContainer> groupContainers;

    @Override
    public String getClassType() {
        return classType;
    }

    @Override
    public Container getContainerById(String vmSeqId) {
        if (vmSeqId.equals(id) || vmSeqId.equals(containerId)) {
            return this;
        }
        List<Container> containers = fetchGroupContainers();
        if (containers != null) {
            for (Container container : containers) {
                if (vmSeqId.equals(container.getId()) || vmSeqId.equals(container.getContainerId())) {
                    return container;
                }
            }
        }
        return null;
    }

    @Override
    public void retryFreshMatrixOption() {
        groupContainers = null;
        if (matrixControlOption != null) {
            matrixControlOption.setFinishCount(null);
            matrixControlOption.setTotalCount(null);
        }
    }

    @Override
    public List<Container> fetchGroupContainers() {
        return groupContainers != null ? List.copyOf(groupContainers) : null;
    }

    @Override
    public Map<String, String> fetchMatrixContext() {
        return matrixContext;
    }

    @Override
    public boolean containerEnabled() {
        return jobControlOption == null || jobControlOption.getEnable();
    }

    @Override
    public void setContainerEnable(boolean enable) {
        jobControlOption = jobControlOption != null ? jobControlOption.setEnable(enable) : new JobControlOption().setEnable(enable);
    }

    @Override
    public void transformCompatibility() {
        if (jobControlOption != null && (jobControlOption.getTimeoutVar() == null || jobControlOption.getTimeoutVar().isEmpty())) {
            jobControlOption.setTimeoutVar(String.valueOf(jobControlOption.getTimeout()));
        }
        if (mutexGroup != null && (mutexGroup.getTimeoutVar() == null || mutexGroup.getTimeoutVar().isEmpty())) {
            mutexGroup.setTimeoutVar(String.valueOf(mutexGroup.getTimeout()));
        }
        super.transformCompatibility();
    }

    @Override
    public String getTemplate() {
        return null;
    }

    @Override
    public String getRef() {
        return null;
    }

    @Override
    public Map<String, String> getVariables() {
        return null;
    }

    @Override
    public void setTemplate(String template) {

    }

    @Override
    public void setRef(String ref) {

    }

    @Override
    public void setVariables(Map<String, String> variables) {

    }
}
