package cd.casic.ci.common.pipeline.pojo.agent;

import cd.casic.ci.common.pipeline.type.BuildType;
import cd.casic.ci.common.pipeline.utils.EnvUtils;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Getter;
import lombok.Setter;

import java.util.Map;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.type.agent
 * @Project：ops-pro
 * @name：ThirdPartyAgentIDDispatchType
 * @Date：2025/03/27 10:48
 * @Filename：ThirdPartyAgentIDDispatchType
 * @description：Todo
 */
@Getter
@Setter
public class ThirdPartyAgentIDDispatchType extends ThirdPartyAgentDispatch {
    @JsonProperty("value")
    private String displayName;
    private String workspace;
    private AgentType agentType = AgentType.NAME;
    private ThirdPartyAgentDockerInfo dockerInfo;
    private ReusedInfo reusedInfo;

    public ThirdPartyAgentIDDispatchType(String displayName, String workspace, ThirdPartyAgentDockerInfo dockerInfo, ReusedInfo reusedInfo) {
        super(displayName, AgentType.NAME, workspace, dockerInfo, reusedInfo);
        this.displayName = displayName;
        this.workspace = workspace;
        this.dockerInfo = dockerInfo;
        this.reusedInfo = reusedInfo;
    }

    @Override
    public void cleanDataBeforeSave() {
        this.displayName = this.displayName.trim();
        this.workspace = this.workspace != null ? this.workspace.trim() : null;
    }

    @Override
    public void replaceField(Map<String, String> variables) {
        displayName = EnvUtils.parseEnv(displayName, variables);
        if (workspace != null && !workspace.isBlank()) {
            workspace = EnvUtils.parseEnv(workspace, variables);
        }
        if (dockerInfo != null) {
            dockerInfo.replaceField(variables);
        }
    }

    @Override
    public BuildType buildType() {
        return BuildType.THIRD_PARTY_AGENT_ID;
    }
}
