package cd.casic.ci.common.pipeline.pojo.element.agent;

import cd.casic.ci.common.pipeline.NameAndValue;
import cd.casic.ci.common.pipeline.enums.BuildScriptType;
import cd.casic.ci.common.pipeline.pojo.element.Element;
import cd.casic.ci.common.pipeline.pojo.element.ElementAdditionalOptions;
import cd.casic.ci.common.pipeline.pojo.transfer.PreStep;
import com.alibaba.fastjson.JSONObject;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.net.URLEncoder;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.pojo.element.agent
 * @Project：ops-pro
 * @name：LinuxScriptElement
 * @Date：2025/03/26 16:37
 * @Filename：LinuxScriptElement
 * @description：Todo
 */
@Data
@NoArgsConstructor
@Schema(title = "脚本任务（linux和macOS环境）", description = LinuxScriptElement.classType)
public class LinuxScriptElement extends Element {
    public static final String classType = "linuxScript";

    @Schema(title = "任务名称", required = true)
    private String name = "执行Linux脚本";
    @Schema(title = "id", required = false)
    private String id;
    @Schema(title = "状态", required = false)
    private String status;
    @Schema(title = "用户自定义ID", required = false)
    private String stepId;
    @Schema(title = "用户自定义环境变量（插件运行时写入环境）", required = false)
    private List<NameAndValue> customEnv;
    @Schema(title = "FAQ url链接", required = false)
    private String errorFAQUrl;
    @Schema(title = "脚本类型", required = true)
    private BuildScriptType scriptType;
    @Schema(title = "脚本内容", required = true)
    private String script;
    @Schema(title = "某次执行为非0时（失败）是否继续执行脚本", required = false)
    private Boolean continueNoneZero;
    @Schema(title = "启用脚本执行失败时归档的文件", required = false)
    private Boolean enableArchiveFile = false;
    @Schema(title = "脚本执行失败时归档的文件", required = false)
    private String archiveFile;
    @Schema(title = "附加参数", required = false)
    private ElementAdditionalOptions additionalOptions;

    @Override
    public Map<String, Object> genTaskParams() {
        Map<String, Object> mutableMap = super.genTaskParams();
        try {
            mutableMap.put("script", URLEncoder.encode(script, "UTF-8"));
        } catch (Exception e) {
            mutableMap.put("script", script);
        }
        return mutableMap;
    }

    public PreStep transferYaml(JSONObject defaultValue) {
        Map<String, Object> res = new HashMap<>();
        res.put("script", script);
        if (continueNoneZero != null && continueNoneZero) {
            res.put("continueNoneZero", true);
        }
        if (enableArchiveFile != null && enableArchiveFile && archiveFile != null) {
            res.put("enableArchiveFile", true);
            res.put("archiveFile", archiveFile);
        }
        return new PreStep(
                name,
                stepId,
                getAtomCode() + "@" + getVersion(),
                res
        );
    }

    @Override
    public String getClassType() {
        return classType;
    }
}
