package cd.casic.pipeline.environment.pojo.enums;

import lombok.Getter;

import java.text.MessageFormat;
import java.util.Locale;

/**
 * @author by mianbin
 * @Classname NodeStatus
 * @Description TODO
 * @Date 2025/4/8 20:00
 */
@Getter
public enum NodeStatus {
    NORMAL("normal"), // 正常
    ABNORMAL("abnormal"), // 异常（节点类型：构建 - 蓝盾agent异常，部署 - GSE agent异常）
    NOT_INSTALLED("notInstall"), // 未安装agent
    DELETED("deleted"), // 已删除
    LOST("lost"), // 失联
    CREATING("creating"), // 正在创建中
    RUNNING("running"), // 安装Agent
    STARTING("starting"), // 正在开机中
    STOPPING("stopping"), // 正在关机中
    STOPPED("stopped"), // 已关机
    RESTARTING("restarting"), // 正在重启中
    DELETING("deleting"), // 正在销毁中
    BUILDING_IMAGE("buildingImage"), // 正在制作镜像中
    BUILD_IMAGE_SUCCESS("buildImageSuccess"), // 制作镜像成功
    BUILD_IMAGE_FAILED("buildImageFailed"), // 制作镜像失败
    NOT_IN_CC("notInCC"), // CC中不存在（仅部署类型节点）
    NOT_IN_CMDB("notInCmdb"), // CMDB中不存在（仅部署类型节点）
    UNKNOWN("unknown"); // 未知

    private final String statusName;

    NodeStatus(String statusName) {
        this.statusName = statusName;
    }

    public static String getStatusName(String status) {
        for (NodeStatus nodeStatus : values()) {
            if (nodeStatus.name().equals(status.toUpperCase(Locale.getDefault()))) {
                return MessageFormat.format("envNodeStatus.{0}", nodeStatus.name());
            }
        }
        return MessageFormat.format("envNodeStatus.{0}", UNKNOWN.name());
    }

    public static NodeStatus parseByName(String name) {
        for (NodeStatus nodeStatus : values()) {
            if (nodeStatus.name().equals(name)) {
                return nodeStatus;
            }
        }
        return UNKNOWN;
    }

    public static NodeStatus parseByStatusName(String statusName) {
        for (NodeStatus nodeStatus : values()) {
            if (nodeStatus.getStatusName().equals(statusName)) {
                return nodeStatus;
            }
        }
        return UNKNOWN;
    }
}
