package cd.casic.ci.log.service;

import cd.casic.ci.log.log.ILogEvent;
import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.framework.commons.util.spring.SpringUtils;
import cd.casic.ci.log.dal.pojo.enums.LogErrorCodeEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * @author by mianbin
 * @Classname BuildLogPrintService
 * @Description 日志打印
 * @Date 2025/3/20 14:57
 */
@Slf4j
@Service
public class BuildLogPrintService {
    // todo, 这里线程池都是使用的默认参数，之后可以载入 logServiceConfig 在配置文件里面对参数进行修改
    private ThreadPoolExecutor logExecutorService = new ThreadPoolExecutor(
            100,
            100,
            0L,
            TimeUnit.MILLISECONDS,
            new LinkedBlockingQueue(1000));

    public void dispatchEvent(ILogEvent event) {
        SpringUtils.publishEvent(event);
    }

    // 异步打印日志
    public CommonResult<Boolean> asyncDispatchEvent(ILogEvent event) {
        // todo 检测配置文件，如果没有开启日志保存,进行的一些处理
        try {
            // 从线程池里面拿一个线程出来处理这个事件
            logExecutorService.execute(() -> {
                System.out.println("BuildLogPrintService[ " + event.getBuildId() + "] " + event.toString());
//                dispatchEvent(event);
            });
            return CommonResult.success(true);
        } catch (RejectedExecutionException e) {
            // 队列满时的处理逻辑
            log.error(
                    "BKSystemErrorMonitor | BuildLogPrintService[ " + event.getBuildId() + "] " +
                            "asyncDispatchEvent failed with queue tasks exceed the limit", e);
            return CommonResult.error(509, LogErrorCodeEnum.PRINT_QUEUE_LIMIT.formatErrorMessage);
        }
    }
}
