package cd.casic.ci.process.process.service.stage.impl;


import cd.casic.ci.common.pipeline.req.stage.StageCreateReq;
import cd.casic.ci.common.pipeline.req.stage.StageUpdateReq;
import cd.casic.ci.common.pipeline.req.task.TaskCreateReq;
import cd.casic.ci.common.pipeline.resp.stage.StageResp;
import cd.casic.ci.process.process.dal.pipeline.PipStageDao;
import cd.casic.ci.process.process.dataObject.stage.PipStage;
import cd.casic.ci.process.process.dataObject.task.PipTask;
import cd.casic.ci.process.process.service.stage.StageService;
import cd.casic.ci.process.process.service.task.TaskService;
import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.framework.commons.exception.enums.GlobalErrorCodeConstants;
import cd.casic.framework.commons.util.object.BeanUtils;
import cd.casic.framework.commons.util.util.WebFrameworkUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONObject;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class StageServiceImpl extends ServiceImpl<PipStageDao, PipStage> implements StageService {
    @Resource
    private TaskService taskService;
    @Resource
    private PipStageDao stageDao;
    @Transactional(rollbackFor = Exception.class)
    @Override
    public String createStagesOrTask(StageCreateReq stageReq) {
        String firstStageId = stageReq.getStageId();
        Integer stageSort = stageReq.getStageSort();
        TaskCreateReq task = stageReq.getTask();
        String taskName = task.getTaskName();
        String secondStageId = task.getStageId();
        JSONObject taskProperties = task.getTaskProperties();
        String taskType = task.getTaskType();
        Integer taskSort = task.getTaskSort();
        String pipelineId = task.getPipelineId();
        Long loginUserId = WebFrameworkUtils.getLoginUserId();
        PipStage firstStage = null;
        // 判断是否需要新建阶段,如果没有传第一级tageId而传了sort就是需要创建阶段
        if (StringUtils.isEmpty(firstStageId)) {
            // 新建stage
            firstStage = new PipStage();
            firstStage.setStageName("阶段-"+stageSort);
            if (stageSort == null) {
                throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"参数有误");
            }
            firstStage.setStageSort(stageSort);
            firstStage.setCreateTime(LocalDateTime.now());
            firstStage.setParentId("-1");
            firstStage.setPipelineId(pipelineId);

            firstStage.setCreateUserId(loginUserId);
            firstStage.setUpdateUserId(loginUserId);
            firstStage.setUpdateTime(LocalDateTime.now());
            PipStage stageQuery = new PipStage();
            stageQuery.setPipelineId(pipelineId);
            stageQuery.setParentId("-1");
            List<PipStage> otherStageList = getPipStageList(stageQuery);
            if (!CollectionUtils.isEmpty(otherStageList)) {
                for (PipStage stage : otherStageList) {
                    if (stageSort<=stage.getStageSort()) {
                        stage.setStageSort(stage.getStageSort()+1);
                    }
                }
                updateBatchById(otherStageList);
            }
            save(firstStage);
        } else {
            PipStage stageQuery = new PipStage();
            stageQuery.setStageId(firstStageId);
            List<PipStage> pipStageList = getPipStageList(stageQuery);
            if (!CollectionUtils.isEmpty(pipStageList)) {
                firstStage = pipStageList.get(0);
            }
        }
        if (firstStage == null) {
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"创建节点失败");
        }
        PipStage secondStage = null;
        if (secondStageId==null||taskSort==null) {
            // 添加并行节点
            secondStage = new PipStage();
            PipStage stageQuery = new PipStage();
            stageQuery.setParentId(firstStageId);
            stageQuery.setPipelineId(pipelineId);
            List<PipStage> pipStageList = getPipStageList(stageQuery);
            if (CollectionUtils.isEmpty(pipStageList)) {
                secondStage.setStageSort(1);
            } else {
                secondStage.setStageSort(pipStageList.size()+1);
            }
            secondStage.setCreateTime(LocalDateTime.now());
            secondStage.setUpdateTime(LocalDateTime.now());
            secondStage.setCreateUserId(loginUserId);
            secondStage.setParentId(firstStageId);
            secondStage.setPipelineId(pipelineId);
            secondStage.setUpdateUserId(loginUserId);
            save(secondStage);
            taskSort=1;
            secondStageId = secondStage.getStageId();
        } else {
            secondStage = getById(secondStageId);
        }
        if (secondStage == null) {
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"创建节点失败");
        }
        // 保存task
        PipTask pipTask = new PipTask();
        pipTask.setTaskType(taskType);
        pipTask.setTaskSort(taskSort);
        pipTask.setStageId(secondStageId);
        pipTask.setTaskName(taskName);
        pipTask.setCreateTime(LocalDateTime.now());
        pipTask.setTaskProperties(taskProperties);
        pipTask.setPipelineId(pipelineId);
        pipTask.setUpdateTime(LocalDateTime.now());
        pipTask.setCreateUserId(loginUserId);
        pipTask.setUpdateUserId(loginUserId);
        taskService.save(pipTask);
        return pipTask.getTaskId();
    }

    @Override
    public List<StageResp> findAllStagesTask(String pipelineId) {
        //获取流水线主节点
        List<PipStage> stageMainStage = findAllMainStage(pipelineId);
        if (stageMainStage.isEmpty()){
            return Collections.emptyList();
        }
        List<StageResp> list = new ArrayList<>();
        for (PipStage stage : stageMainStage) {
            String stagesId = stage.getStageId();
            //获取从节点
            List<PipStage> allStageStage = findOtherStageNoTask(stagesId);
            List<StageResp> stageRespList = allStageStage.stream().map(it -> {
                StageResp stageResp = new StageResp();
                BeanUtils.copyProperties(it, stageResp);
                return stageResp;
            }).toList();
            StageResp stageResp = new StageResp();
            BeanUtils.copyProperties(stage,stageResp);
            stageResp.setStageList(stageRespList);
            list.add(stageResp);
        }
        list.sort(Comparator.comparing(StageResp::getStageSort));
        return list;
    }

    @Override
    public void deleteStagesOrTask(String taskId) {
        PipTask taskQuery = new PipTask();
        taskQuery.setTaskId(taskId);
        List<PipTask> taskList = taskService.getTask(taskQuery);
        if (CollectionUtils.isEmpty(taskList)) {
            return;
        }
        PipTask task = taskList.get(0);
        taskService.removeById(taskId);
        // 查询上一级stage下有无其他task 没有则连着stage删除
        String stageId = task.getStageId();
        String pipelineId = task.getPipelineId();
        taskQuery.setTaskId("");
        taskQuery.setStageId(stageId);
        List<PipTask> otherTask = taskService.getTask(task);
        if (CollectionUtils.isEmpty(otherTask)) {
            // 删除当前task的父stage，然后判断父stage的父级有无其他子集如果没有就继续删除当前阶段

            PipStage stageQuery = new PipStage();
            stageQuery.setPipelineId(stageId);
            List<PipStage> currStageList = getPipStageList(stageQuery);
            if (CollectionUtils.isEmpty(currStageList)) {
                throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"stage不存在");
            }
            PipStage currStage = currStageList.get(0);
            deleteStages(stageId);
            String parentId = currStage.getParentId();
            stageQuery.setParentId(parentId);
            stageQuery.setStageId(null);
            // 查询同阶段其他二级stage，如果不存在其他stage则删除阶段stage并整理sort值
            List<PipStage> otherStageList = getPipStageList(stageQuery);
            if (CollectionUtils.isEmpty(otherStageList)) {
                //没有其他并行路径就需要删除当前阶段
                deleteStages(parentId);
            } else {
                for (PipStage stage : otherStageList) {
                    if (currStage.getStageSort()<stage.getStageSort()) {
                        stage.setStageSort(stage.getStageSort()-1);
                    }
                }
            }
        } else {
            for (PipTask pipTask : otherTask) {
                if (task.getTaskSort()<pipTask.getTaskSort()) {
                    pipTask.setTaskSort(pipTask.getTaskSort()-1);
                }
            }
        }

    }

    @Override
    public void deleteAllStagesOrTask(String pipelineId) {
        // TODO 删除流水线，不知道要不要
    }

    public List<PipStage> getPipStageList(PipStage pipStage){
        LambdaQueryWrapper<PipStage> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(StringUtils.isNotEmpty(pipStage.getStageId()),PipStage::getStageId,pipStage.getStageId());
        wrapper.eq(StringUtils.isNotEmpty(pipStage.getPipelineId()),PipStage::getPipelineId,pipStage.getPipelineId());
        wrapper.eq(StringUtils.isNotEmpty(pipStage.getParentId()),PipStage::getParentId,pipStage.getParentId());
        return stageDao.selectList(wrapper);
    }

    @Override
    public List<PipStage> findAllMainStage(String pipelineId) {
        LambdaQueryWrapper<PipStage> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(PipStage::getPipelineId,pipelineId);
        wrapper.eq(PipStage::getParentId,"-1");
        return stageDao.selectList(wrapper);
    }
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateStagesTask(StageUpdateReq stage) {
        String stageId = stage.getStageId();
        String stageName = stage.getStageName();
        Integer currStageSort = stage.getStageSort();
        Long loginUserId = WebFrameworkUtils.getLoginUserId();
        PipStage updateStage = getById(stageId);
        if (updateStage==null) {
            return;
        }

        updateStage.setUpdateUserId(loginUserId);
        updateStage.setUpdateTime(LocalDateTime.now());
        if (currStageSort == null) {
            updateStage.setStageName(stageName);
            updateById(updateStage);
        } else{
            Integer oldStageSort = updateStage.getStageSort();
            if (oldStageSort.equals(currStageSort)) {
                return;
            }
            PipStage query = new PipStage();
            query.setPipelineId(updateStage.getPipelineId());
            query.setParentId("-1");
            List<PipStage> stageList = getPipStageList(query);
            if (oldStageSort<currStageSort) {
                // 往右移动
                for (PipStage pipStage : stageList) {
                    if (!pipStage.getStageId().equals(updateStage.getStageId())) {
                        if (pipStage.getStageSort()>oldStageSort&&pipStage.getStageSort()<=currStageSort) {
                            pipStage.setStageSort(pipStage.getStageSort()-1);
                        }
                    }
                }
            } else {
                // 往左移动
                for (PipStage pipStage : stageList) {
                    if (!pipStage.getStageId().equals(updateStage.getStageId())) {
                        if (pipStage.getStageSort()<oldStageSort&&pipStage.getStageSort()>=currStageSort) {
                            pipStage.setStageSort(pipStage.getStageSort()+1);
                        }
                    }
                }
            }
            updateBatchById(stageList);
        }

    }

    @Override
    public List<String> validStagesMustField(String pipelineId) {
        return null;
    }



    @Override
    public void updateStages(PipStage stage) {
        stageDao.updateById(stage);
    }

    @Override
    public void deleteStages(String stageId) {
        stageDao.deleteById(stageId);
    }

    @Override
    @Transactional
    public void copyStage(String stageId) {
        // 查询当前stage（阶段下所有）
        PipStage firstStage = getById(stageId);
        Long loginUserId = WebFrameworkUtils.getLoginUserId();
        List<PipStage> allMainStage = findAllMainStage(firstStage.getPipelineId());
        List<PipStage> updateStageList = new ArrayList<>(allMainStage.size());
        for (PipStage stage : allMainStage) {
            if (stage.getStageSort()>firstStage.getStageSort()) {
                stage.setStageSort(stage.getStageSort()+1);
                updateStageList.add(stage);
            }
        }
        // 保存复制后的阶段，除了id和sort其他都一样
        firstStage.setStageId(null);
        firstStage.setStageSort(firstStage.getStageSort()+1);
        save(firstStage);
        updateBatchById(updateStageList);
        // 查询阶段下所有分支
        List<PipStage> secondStageList = findSecondStage(stageId);
        if (!CollectionUtils.isEmpty(secondStageList)) {
            List<String> stageIdList = secondStageList.stream().map(PipStage::getStageId).toList();
            List<PipTask> taskList = taskService.listByIds(stageIdList);
            Map<String, List<PipTask>> stageIdMap = taskList.stream().collect(Collectors.groupingBy(PipTask::getStageId));
            for (PipStage secondStage : secondStageList) {
                secondStage.setCreateUserId(loginUserId);
                secondStage.setCreateTime(LocalDateTime.now());
                secondStage.setUpdateTime(LocalDateTime.now());
                secondStage.setUpdateUserId(loginUserId);
                secondStage.setParentId(firstStage.getStageId());
                String secondStageId = secondStage.getStageId();
                List<PipTask> childTask = stageIdMap.get(secondStageId);
                secondStage.setStageId(null);
                save(secondStage);
                for (PipTask pipTask : childTask) {
                    pipTask.setStageId(secondStage.getStageId());
                    pipTask.setTaskId(null);
                    pipTask.setCreateUserId(loginUserId);
                    pipTask.setCreateTime(LocalDateTime.now());
                    pipTask.setUpdateTime(LocalDateTime.now());
                    pipTask.setUpdateUserId(loginUserId);
                }
                taskService.saveBatch(childTask);
            }
        }

    }

    public List<PipStage> findOtherStageNoTask(String stagesId){
        List<PipStage> otherStage = findSecondStage(stagesId);
        List<PipStage> list = new ArrayList<>();
        List<String> stageIdList = otherStage.stream().map(PipStage::getStageId).toList();
        Map<String, List<PipTask>> stageIdTaskMap = taskService.listByIds(stageIdList).stream().collect(Collectors.groupingBy(PipTask::getStageId));
        for (PipStage stage : otherStage) {
            //获取阶段配置及任务
            String otherId = stage.getStageId();
            List<PipTask> allStageTask = stageIdTaskMap.get(otherId);
            stage.setTaskValues(allStageTask);
            list.add(stage);
        }
        return list;
    }
    // 获取主stage（阶段）下的子stage
    @Override
    public List<PipStage> findSecondStage(String stageId){
        LambdaQueryWrapper<PipStage> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(PipStage::getParentId,stageId);
        return stageDao.selectList(wrapper);
    }
}
