package cd.casic.ci.process.process.service.task.impl;

import cd.casic.ci.common.pipeline.annotation.Plugin;
import cd.casic.ci.common.pipeline.resp.task.TasksResp;
import cd.casic.ci.process.process.dal.pipeline.PipStageDao;
import cd.casic.ci.process.process.dal.pipeline.PipTaskDao;
import cd.casic.ci.process.process.dataObject.task.PipTask;
import cd.casic.ci.process.process.service.stage.StageService;
import cd.casic.ci.process.process.service.task.TaskService;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import jakarta.annotation.PostConstruct;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.ClassPathScanningCandidateComponentProvider;
import org.springframework.core.type.filter.AnnotationTypeFilter;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;


import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import java.util.Set;


@Service
public class TaskServiceImpl extends ServiceImpl<PipTaskDao, PipTask> implements TaskService {
    private static final String basePackage = "cd.casic.ci.process.process.service";
    private Set<BeanDefinition> candidates;
    @Resource
    private PipTaskDao taskDao;

    @PostConstruct
    public void init(){
        ClassPathScanningCandidateComponentProvider provider =
                new ClassPathScanningCandidateComponentProvider(false);
        provider.addIncludeFilter(new AnnotationTypeFilter(Plugin.class));
        candidates = provider.findCandidateComponents(basePackage);
        for (BeanDefinition candidate : candidates) {

        }

    }
    @Override
    public void taskTypeExist(String taskType) {

    }

    @Override
    public List<PipTask> getTask(PipTask task) {
        LambdaQueryWrapper<PipTask> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(StringUtils.isNotEmpty(task.getTaskId()),PipTask::getTaskId,task.getTaskId());
        wrapper.eq(StringUtils.isNotEmpty(task.getPipelineId()),PipTask::getPipelineId,task.getPipelineId());
        wrapper.eq(StringUtils.isNotEmpty(task.getStageId()),PipTask::getStageId,task.getStageId());
        return taskDao.selectList(wrapper);
    }
    @Override
    public TasksResp getRespById(String taskId){
        PipTask pipTask = new PipTask();
        pipTask.setTaskId(taskId);
        List<PipTask> taskList = getTask(pipTask);
        if (!CollectionUtils.isEmpty(taskList)) {
            return null;
        }
        TasksResp tasksResp = new TasksResp();
        BeanUtils.copyProperties(taskList.get(0),tasksResp);
        return tasksResp;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void copyTask(String taskId) {
        PipTask sourceTask = getById(taskId);
        if (sourceTask == null) {
            return;
        }
        String stageId = sourceTask.getStageId();
        PipTask query = new PipTask();
        query.setStageId(stageId);
        List<PipTask> sameLevelTaskList = getTask(query);
        if (CollectionUtils.isEmpty(sameLevelTaskList)) {
            return;
        }
        List<PipTask> updateList = new ArrayList<>(sameLevelTaskList.size());
        for (PipTask pipTask : sameLevelTaskList) {
            if (!pipTask.getTaskId().equals(sourceTask.getTaskId())) {
                if (pipTask.getTaskSort()>sourceTask.getTaskSort()) {
                    pipTask.setTaskSort(pipTask.getTaskSort()+1);
                    updateList.add(pipTask);
                }
            }
        }
        sourceTask.setTaskSort(sourceTask.getTaskSort()+1);
        sourceTask.setTaskId(null);
        save(sourceTask);
        if (!CollectionUtils.isEmpty(updateList)) {
            updateBatchById(updateList);
        }
    }
}
