package cd.casic.pipeline.store.api.common;

import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.pipeline.store.pojo.common.category.Category;
import cd.casic.pipeline.store.pojo.common.category.CategoryRequest;
import cd.casic.pipeline.store.pojo.common.enums.StoreTypeEnum;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Tag(name = "OP_STORE_CATEGORY", description = "OP-STORE-范畴")
@RestController
@RequestMapping("/op/store/category")
public interface OpCategoryResource {

    @Operation(summary = "添加范畴")
    @PostMapping("/types/{categoryType}")
    CommonResult<Boolean> add(
            @Parameter(description = "类别", required = true)
            @PathVariable("categoryType")
            StoreTypeEnum categoryType,
            @Parameter(description = "范畴信息请求报文体", required = true)
            CategoryRequest categoryRequest
    );

    @Operation(summary = "更新范畴信息")
    @PutMapping("/types/{categoryType}/ids/{id}")
    CommonResult<Boolean> update(
            @Parameter(description = "类别", required = true)
            @PathVariable("categoryType")
            StoreTypeEnum categoryType,
            @Parameter(description = "范畴ID", required = true)
            @PathVariable("id")
            String id,
            @Parameter(description = "范畴信息请求报文体", required = true)
            CategoryRequest categoryRequest
    );

    @Operation(summary = "获取所有范畴信息")
    @GetMapping("/types/{categoryType}")
    CommonResult<List<Category>> listAllCategorys(
            @Parameter(description = "类别", required = true)
            @PathVariable("categoryType")
            StoreTypeEnum categoryType
    );

    @Operation(summary = "根据ID获取范畴信息")
    @GetMapping("/{id}")
    CommonResult<Category> getCategoryById(
            @Parameter(description = "范畴ID", required = true)
            @RequestParam("id")
            String id
    );

    @Operation(summary = "根据ID删除范畴信息")
    @DeleteMapping("/{id}")
    CommonResult<Boolean> deleteCategoryById(
            @Parameter(description = "范畴ID", required = true)
            @PathVariable("id")
            String id
    );
}