package cd.casic.pipeline.store.api.common;

import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.pipeline.store.pojo.common.*;
import cd.casic.pipeline.store.pojo.common.enums.StoreSortTypeEnum;
import cd.casic.pipeline.store.pojo.common.enums.StoreTypeEnum;
import cd.casic.pipeline.store.pojo.common.publication.StoreApproveReleaseRequest;
import cd.casic.pipeline.store.pojo.common.version.StoreDeskVersionItem;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import org.springframework.web.bind.annotation.*;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;
import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID_DEFAULT_VALUE;

@Tag(name = "OP_STORE_COMPONENT", description = "研发商店-OP-组件管理")
@RestController
@RequestMapping("/op/store/components")
public interface OpStoreComponentResource {

    @Operation(summary = "审核组件发布")
    @PutMapping("/{storeId}/release/approve")
    CommonResult<Boolean> approveComponentRelease(
            @Parameter(description = "用户ID", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件ID", required = true)
            @PathVariable("storeId")
            String storeId,
            @Parameter(description = "审核组件发布请求请求报文")
            StoreApproveReleaseRequest storeApproveReleaseRequest
    );

    @Operation(summary = "获取组件列表")
    @GetMapping("/types/{storeType}/component/list")
    PageResult<MyStoreComponent> listComponents(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            String storeType,
            @Parameter(description = "名称", required = false)
            @RequestParam("name")
            String name,
            @Parameter(description = "类型", required = false)
            @RequestParam("type")
            String type,
            @Parameter(description = "是否处于流程中", required = false)
            @RequestParam("processFlag")
            Boolean processFlag,
            @Parameter(description = "分类", required = false)
            @RequestParam("classifyCode")
            String classifyCode,
            @Parameter(description = "应用范畴，多个用逗号分隔", required = false)
            @RequestParam("categoryCodes")
            String categoryCodes,
            @Parameter(description = "功能标签，多个用逗号分隔", required = false)
            @RequestParam("labelCodes")
            String labelCodes,
            @Parameter(description = "排序", required = false)
            @RequestParam("sortType")
            StoreSortTypeEnum sortType,
            @Parameter(description = "页码", required = true)
            @RequestParam("page")
            Integer page,
            @Parameter(description = "每页数量", required = true)
            @RequestParam("pageSize")
            Integer pageSize
    );

    @Operation(summary = "根据组件标识获取组件版本列表")
    @GetMapping("/types/{storeType}/codes/{storeCode}/component/version/list")
    PageResult<StoreDeskVersionItem> getComponentVersionsByCode(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            String storeType,
            @Parameter(description = "组件代码", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "页码", required = true)
            @RequestParam("page")
            Integer page,
            @Parameter(description = "每页数量", required = true)
            @RequestParam("pageSize")
            Integer pageSize
    );

    @Operation(summary = "根据组件ID获取组件详情")
    @GetMapping("/types/{storeType}/ids/{storeId}/component/detail")
    CommonResult<StoreDetailInfo> getComponentDetailInfoById(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            String storeType,
            @Parameter(description = "组件ID", required = true)
            @PathVariable("storeId")
            String storeId
    );

    @Operation(summary = "更新组件基本信息")
    @PutMapping("/types/{storeType}/codes/{storeCode}/component/base/info/update")
    CommonResult<Boolean> updateComponentBaseInfo(
            @Parameter(description = "用户ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            String storeType,
            @Parameter(description = "组件代码", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "组件基本信息修改请求报文体", required = true)
            StoreBaseInfoUpdateRequest storeBaseInfoUpdateRequest
    );

    @Operation(summary = "删除工作台组件")
    @DeleteMapping("/desk/types/{storeType}/codes/{storeCode}/component/delete")
    CommonResult<Boolean> deleteComponent(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            String storeType,
            @Parameter(description = "组件代码", required = true)
            @PathVariable("storeCode")
            String storeCode
    );

    @Operation(summary = "下架组件")
    @PutMapping("/component/offline")
    CommonResult<Boolean> offlineComponent(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "工作台-下架组件请求报文体", required = true)
            @Valid
            StoreOfflineRequest storeOfflineRequest
    );

    @Operation(summary = "更新组件已安装包文件sha1摘要值")
    @PutMapping("/types/{storeType}/codes/{storeCode}/versions/{version}/component/installed/pkg/file/sha/update")
    CommonResult<Boolean> updateComponentInstalledPkgFileShaContent(
            @Parameter(description = "用户ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            StoreTypeEnum storeType,
            @Parameter(description = "组件代码", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "组件版本", required = true)
            @PathVariable("version")
            String version,
            @Parameter(description = "更新组件已安装包文件sha1摘要值请求报文", required = true)
            InstalledPkgFileShaContentRequest installedPkgFileShaContentRequest
    );
}