package cd.casic.pipeline.store.api.common;

import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.pipeline.store.pojo.common.env.StorePkgRunEnvRequest;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import org.springframework.web.bind.annotation.*;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;


@Tag(name = "OP_STORE_PKG_ENVS", description = "安装包环境信息")
@RestController
@RequestMapping("/op/store/pkg/envs")
public interface OpStorePkgEnvInfoResource {

    @Operation(summary = "新增环境变量")
    @PostMapping("/create")
    CommonResult<Boolean> create(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "环境变量请求报文体", required = true)
            @Valid
            StorePkgRunEnvRequest storePkgRunEnvRequest
    );

    @Operation(summary = "更新环境变量")
    @PostMapping("/ids/{id}/update")
    CommonResult<Boolean> update(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "主键ID", required = true)
            @PathVariable("id")
            String id,
            @Parameter(description = "安装包运行时环境信息请求报文体", required = true)
            @Valid
            StorePkgRunEnvRequest storePkgRunEnvRequest
    );

    @Operation(summary = "删除环境变量")
    @DeleteMapping("/ids/{id}/delete")
    CommonResult<Boolean> delete(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "主键ID", required = true)
            @PathVariable("id")
            String id,
            @Parameter(description = "安装包运行时环境信息请求报文体", required = true)
            @Valid
            StorePkgRunEnvRequest storePkgRunEnvRequest
    );
}