package cd.casic.pipeline.store.api.common;

import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.pipeline.store.pojo.common.env.StoreEnvChangeLogInfo;
import cd.casic.pipeline.store.pojo.common.env.StoreEnvVarInfo;
import cd.casic.pipeline.store.pojo.common.env.StoreEnvVarRequest;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;


@Tag(name = "USER_STORE_ENV_VAR", description = "环境变量")
@RestController
@RequestMapping("/user/store/env/var")
public interface UserStoreEnvVarResource {

    @Operation(summary = "新增环境变量")
    @PostMapping("/create")
    CommonResult<Boolean> create(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "环境变量请求报文体", required = true)
            @Valid
            StoreEnvVarRequest storeEnvVarRequest
    );

    @Operation(summary = "更新环境变量")
    @PostMapping("/update")
    CommonResult<Boolean> update(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "变量ID", required = true)
            @RequestParam("variableId")
            String variableId,
            @Parameter(description = "环境变量请求报文体", required = true)
            @Valid
            StoreEnvVarRequest storeEnvVarRequest
    );

    @Operation(summary = "删除环境变量")
    @DeleteMapping("/types/{storeType}/codes/{storeCode}")
    CommonResult<Boolean> delete(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            String storeType,
            @Parameter(description = "组件标识", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "生效范围 TEST：测试 PRD：正式 ALL：所有", required = true)
            @RequestParam("scope")
            String scope,
            @Parameter(description = "环境变量名称集合，用\",\"分隔进行拼接（如1,2,3）", required = true)
            @RequestParam("varNames")
            String varNames
    );

    @Operation(summary = "获取最新的环境变量列表")
    @GetMapping("/latest/types/{storeType}/codes/{storeCode}")
    CommonResult<List<StoreEnvVarInfo>> getLatestEnvVarList(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            String storeType,
            @Parameter(description = "组件标识", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "生效范围 TEST：测试 PRD：正式 ALL：所有", required = false)
            @RequestParam("scope")
            String scope,
            @Parameter(description = "变量名", required = false)
            @RequestParam("varName")
            String varName
    );

    @Operation(summary = "获取环境变量变更记录列表")
    @GetMapping("/change/log/types/{storeType}/codes/{storeCode}/vars/{varName}")
    CommonResult<List<StoreEnvChangeLogInfo>> getEnvVarChangeLogList(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            String storeType,
            @Parameter(description = "组件标识", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "生效范围 TEST：测试 PRD：正式 ALL：所有", required = true)
            @RequestParam("scope")
            String scope,
            @Parameter(description = "变量名", required = true)
            @PathVariable("varName")
            String varName
    );
}