package cd.casic.pipeline.store.api.template;


import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.pipeline.store.pojo.template.ApproveReq;
import cd.casic.pipeline.store.pojo.template.OpTemplateResp;
import cd.casic.pipeline.store.pojo.template.enums.OpTemplateSortTypeEnum;
import cd.casic.pipeline.store.pojo.template.enums.TemplateStatusEnum;
import cd.casic.pipeline.store.pojo.template.enums.TemplateTypeEnum;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;

@Tag(name = "OP_PIPELINE_TEMPLATE", description = "OP-流水线-模版")
@RestController
@RequestMapping("/op/pipeline/template")
public interface OpTemplateResource {

    @Operation(summary = "获取市场模版")
    @GetMapping("/list")
    CommonResult<OpTemplateResp> listTemplates(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "模版名称", required = false)
            @RequestParam("templateName")
            String templateName,
            @Parameter(description = "模版状态", required = false)
            @RequestParam("templateStatus")
            TemplateStatusEnum templateStatus,
            @Parameter(description = "模版类型", required = false)
            @RequestParam("templateType")
            TemplateTypeEnum templateType,
            @Parameter(description = "模版分类", required = false)
            @RequestParam("classifyCode")
            String classifyCode,
            @Parameter(description = "应用范畴", required = false)
            @RequestParam("category")
            String category,
            @Parameter(description = "功能标签", required = false)
            @RequestParam("labelCode")
            String labelCode,
            @Parameter(description = "是否最新", required = false)
            @RequestParam("latestFlag")
            Boolean latestFlag,
            @Parameter(description = "排序", required = false)
            @RequestParam("sortType")
            OpTemplateSortTypeEnum sortType,
            @Parameter(description = "排序", required = false)
            @RequestParam("desc")
            Boolean desc,
            @Parameter(description = "页码", required = false)
            @RequestParam("page")
            Integer page,
            @Parameter(description = "每页数量", required = false)
            @RequestParam("pageSize")
            Integer pageSize
    );

    @Operation(summary = "审核模版")
    @PutMapping("/{templateId}/approve")
    CommonResult<Boolean> approveTemplate(
            @Parameter(description = "用户ID", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "ID", required = true)
            @PathVariable("templateId")
            String templateId,
            @Parameter(description = "审核模版请求报文")
            ApproveReq approveReq
    );
}