package cd.casic.pipeline.store.api.template;


import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.pipeline.store.pojo.common.InstalledProjRespItem;
import cd.casic.pipeline.store.pojo.template.*;
import cd.casic.pipeline.store.pojo.template.enums.MarketTemplateSortTypeEnum;
import cd.casic.pipeline.store.pojo.template.enums.TemplateRdTypeEnum;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;


@Tag(name = "USER_MARKET_TEMPLATE", description = "模板")
@RestController
@RequestMapping("/user/market")
public interface UserTemplateResource {

    @Operation(summary = "模版市场首页")
    @GetMapping("/template/list/main")
    CommonResult<List<MarketTemplateMain>> mainPageList(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "页码", required = false)
            @RequestParam("page")
            Integer page,
            @Parameter(description = "每页数量", required = false)
            @RequestParam("pageSize")
            Integer pageSize
    );

    @Operation(summary = "模版市场搜索模版")
    @GetMapping("/template/list/")
    CommonResult<MarketTemplateResp> list(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "搜索关键字", required = false)
            @RequestParam("keyword")
            String keyword,
            @Parameter(description = "模版分类", required = false)
            @RequestParam("classifyCode")
            String classifyCode,
            @Parameter(description = "应用范畴", required = false)
            @RequestParam("categoryCode")
            String categoryCode,
            @Parameter(description = "功能标签", required = false)
            @RequestParam("labelCode")
            String labelCode,
            @Parameter(description = "评分", required = false)
            @RequestParam("score")
            Integer score,
            @Parameter(description = "研发来源", required = false)
            @RequestParam("rdType")
            TemplateRdTypeEnum rdType,
            @Parameter(description = "排序", required = false)
            @RequestParam("sortType")
            MarketTemplateSortTypeEnum sortType,
            @Parameter(description = "项目ID", required = false)
            @RequestParam("projectCode")
            String projectCode,
            @Parameter(description = "页码", required = false)
            @RequestParam("page")
            Integer page,
            @Parameter(description = "每页数量", required = false)
            @RequestParam("pageSize")
            Integer pageSize
    );

    @Operation(summary = "删除工作台模版")
    @DeleteMapping("/templates/{templateCode}")
    CommonResult<Boolean> delete(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "模板Code", required = true)
            @PathVariable("templateCode")
            String templateCode
    );

    @Operation(summary = "根据ID查看模板详情")
    @GetMapping("/template/templateIds/{templateId}")
    CommonResult<TemplateDetail> getTemplateDetailById(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "模板ID", required = true)
            @PathVariable("templateId")
            String templateId
    );

    @Operation(summary = "根据模板代码查看模板详情")
    @GetMapping("/template/templateCodes/{templateCode}")
    CommonResult<TemplateDetail> getTemplateDetailByCode(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "模板代码", required = true)
            @PathVariable("templateCode")
            String templateCode
    );

    @Operation(summary = "安装模板到项目")
    @PostMapping("/template/install")
    CommonResult<Boolean> installTemplate(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "安装模板到项目请求报文体", required = true)
            InstallTemplateReq installTemplateReq
    );

    @Operation(summary = "安装模板到项目--返回流水线模板信息")
    @PostMapping("/template/install/new")
    CommonResult<InstallTemplateResp> installTemplateNew(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "安装模板到项目请求报文体", required = true)
            InstallTemplateReq installTemplateReq
    );

    @Operation(summary = "根据模板标识获取已安装的项目列表")
    @GetMapping("/template/installedProjects/{templateCode")
    CommonResult<List<InstalledProjRespItem>> getInstalledProjects(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "模版代码", required = true)
            @PathVariable("templateCode")
            String templateCode
    );

    @Operation(summary = "根据用户获取原子工作台模板列表")
    @GetMapping("/desk/template/list")
    PageResult<MyTemplateItem> getMyTemplates(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "模版名称", required = false)
            @RequestParam("templateName")
            String templateName,
            @Parameter(description = "页码", required = true)
            @RequestParam("page")
            Integer page,
            @Parameter(description = "每页数量", required = true)
            @RequestParam("pageSize")
            Integer pageSize
    );
}